import ctypes
import platform

# Define the necessary ctypes structures and constants for mouse control
if platform.system() == 'Windows':
    user32 = ctypes.windll.user32

    # Constants for mouse input
    MOUSEEVENTF_MOVE = 0x0001
    MOUSEEVENTF_LEFTDOWN = 0x0002
    MOUSEEVENTF_LEFTUP = 0x0004
    MOUSEEVENTF_RIGHTDOWN = 0x0008
    MOUSEEVENTF_RIGHTUP = 0x0010

    class MOUSEINPUT(ctypes.Structure):
        _fields_ = [
            ("dx", ctypes.c_long),
            ("dy", ctypes.c_long),
            ("mouseData", ctypes.c_ulong),
            ("dwFlags", ctypes.c_ulong),
            ("time", ctypes.c_ulong),
            ("dwExtraInfo", ctypes.c_ulong)
        ]
    class POINT(ctypes.Structure):
        _fields_ = [("x", ctypes.c_long), ("y", ctypes.c_long)]

    class INPUT(ctypes.Structure):
        _fields_ = [("type", ctypes.c_ulong), ("mi", ctypes.POINTER(MOUSEINPUT))]

# Function to move the mouse to the specified (x, y) coordinates
def move_mouse(x, y):
    """
    Moves the mouse cursor to the specified (x, y) coordinates.
    
    :param x: The x-coordinate to move the mouse to.
    :param y: The y-coordinate to move the mouse to.
    
    Raises:
        ValueError: If x or y are out of screen bounds.
    """
    if platform.system() == 'Windows':
        screen_width = user32.GetSystemMetrics(0)  # Get screen width
        screen_height = user32.GetSystemMetrics(1)  # Get screen height

        # Validate the coordinates
        if not (0 <= x < screen_width) or not (0 <= y < screen_height):
            raise ValueError(f"Invalid mouse position: ({x}, {y}). Must be within (0, 0) to ({screen_width}, {screen_height}).")
        
        user32.SetCursorPos(x, y)
    else:
        raise NotImplementedError("Mouse control is not implemented for this platform.")

# Function to simulate a mouse click
def click_mouse(button="left"):
    """
    Simulates a mouse click.
    
    :param button: 'left' for left click, 'right' for right click.
    
    Raises:
        ValueError: If an invalid button is specified.
    """
    if platform.system() == 'Windows':
        if button == "left":
            user32.mouse_event(MOUSEEVENTF_LEFTDOWN, 0, 0, 0, 0)
            user32.mouse_event(MOUSEEVENTF_LEFTUP, 0, 0, 0, 0)
        elif button == "right":
            user32.mouse_event(MOUSEEVENTF_RIGHTDOWN, 0, 0, 0, 0)
            user32.mouse_event(MOUSEEVENTF_RIGHTUP, 0, 0, 0, 0)
        else:
            raise ValueError("Invalid button specified. Use 'left' or 'right'.")
    else:
        raise NotImplementedError("Mouse control is not implemented for this platform.")

# Additional function to scroll the mouse wheel
def scroll_mouse(amount):
    """
    Scrolls the mouse wheel by the specified amount.
    
    :param amount: Positive value to scroll up, negative to scroll down.
    """
    if platform.system() == 'Windows':
        user32.mouse_event(0x0800, 0, 0, amount, 0)  # 0x0800 = MOUSEEVENTF_WHEEL
    else:
        raise NotImplementedError("Mouse scrolling is not implemented for this platform.")
